/*
 * File: DragObjectsWithKeyEvents.java
 * -----------------------------------
 * This implementation makes it possible to move the last
 * selected object using the arrow keys as well as the mouse.
 */

import acm.graphics.*;
import acm.program.*;
import java.awt.*;
import java.awt.event.*;

/** This class displays a mouse-draggable rectangle and oval */
public class DragObjectsWithKeyEvents extends GraphicsProgram {

/** Initializes the program */
	public void init() {
		GRect rect = new GRect(100, 100, 150, 100);
		rect.setFilled(true);
		rect.setColor(Color.RED);
		add(rect);
		GOval oval = new GOval(300, 115, 100, 70);
		oval.setFilled(true);
		oval.setColor(Color.GREEN);
		add(oval);
		addMouseListeners();
		addKeyListeners();
	}

/** Called on mouse press to record the coordinates of the click */
	public void mousePressed(MouseEvent e) {
		last = new GPoint(e.getPoint());
		gobj = getElementAt(last);
	}

/** Called on mouse drag to reposition the object */
	public void mouseDragged(MouseEvent e) {
		if (gobj != null) {
			gobj.move(e.getX() - last.getX(), e.getY() - last.getY());
			last = new GPoint(e.getPoint());
		}
	}

/** Called on mouse click to move this object to the front */
	public void mouseClicked(MouseEvent e) {
		if (gobj != null) gobj.sendToFront();
	}

/** Called when a key is pressed in the window */
	public void keyPressed(KeyEvent e) {
		if (gobj != null) {
			switch (e.getKeyCode()) {
			  case KeyEvent.VK_UP: gobj.move(0, -1); break;
			  case KeyEvent.VK_DOWN: gobj.move(0, +1); break;
			  case KeyEvent.VK_LEFT: gobj.move(-1, 0); break;
			  case KeyEvent.VK_RIGHT: gobj.move(+1, 0); break;
			}
		}
	}

/* Private instance variables */
	private GObject gobj;           /* The object being dragged */
	private GPoint last;            /* The last mouse position  */

/* Standard Java entry point */
/* This method can be eliminated in most Java environments */
	public static void main(String[] args) {
		new DragObjectsWithKeyEvents().start(args);
	}
}
